
#include "text_t.h"
#include <ctype.h> // for isspace
#include <vector>

void equiv_lookup(text_t& terms) {
  // eventually we'll get these terms out of a config file or something...
  vector<text_t> group;
  vector< vector<text_t> > all_groups;
  
  group.push_back("triene");
  group.push_back("trientine");
  group.push_back("trien");
  group.push_back("\"triethylenetetramine dihydrochloride\"");
  group.push_back("\"triethylene tetramine dihydrochloride\"");
  group.push_back("syprine");
  group.push_back("teta");
  group.push_back(text_t("\"trien 2hcl\""));
  group.push_back(text_t("\"triethylenetetramine 2hcl\""));
  group.push_back(text_t("\"bis aminoethyl ethanediamine dihydrochloride\""));
  all_groups.push_back(group);


  group.clear();
  group.push_back("pkpd");
  group.push_back("pkin");
  group.push_back("pharmocokinetic");
  group.push_back("pharmacodynamic");
  all_groups.push_back(group);


  group.clear();
  group.push_back("safety");
  group.push_back("\"side effects\"");
  group.push_back("toxicity");
  group.push_back("toxicology");
  all_groups.push_back(group);


  group.clear();
  group.push_back("sod");
  group.push_back("\"superoxide dismutase\"");
  group.push_back("\"super oxide dismutase\"");
  all_groups.push_back(group);


  group.clear();
  group.push_back("ceruloplasmin");
  group.push_back("caeruloplasmin");
  all_groups.push_back(group);


  // we always convert to lowercase to do the lookup
  text_t casestemoptions="";
  text_t termproper="";

  text_t::const_iterator hash=terms.end();
  text_t::const_iterator start=terms.begin();
  while (hash!=start) {
    if (*hash=='#') break;
    --hash;
  }
  if (hash!=start) { // we found a hash
    casestemoptions=substr(hash,terms.end());
    termproper=substr(start,hash);
  } else {
    termproper=terms;
  }

  // canonicalise termproper to lowercase for lookup
  lc(termproper.begin(), termproper.end());

  // look up vectors
  vector< vector<text_t> >::const_iterator group_here=all_groups.begin();
  vector< vector<text_t> >::const_iterator group_done=all_groups.end();

  while (group_here != group_done) {
    vector<text_t>::const_iterator syn_here=group_here->begin();
    vector<text_t>::const_iterator syn_done=group_here->end();

    while (syn_here != syn_done) {
      if (*syn_here == termproper) // we found a match
        break;
      ++syn_here;
    }
    if (syn_here != syn_done) { // we found a match in this group
      syn_here=group_here->begin();
      terms="(";
      
      while (syn_here != syn_done) {
        terms+= *syn_here;
        terms+= casestemoptions;
        if (syn_here != syn_done - 1) {
          terms+=" | ";
        }
        ++syn_here;
      }
      terms.push_back(')');
      break;
    }
    ++group_here;
  }

}

void insert_equiv_terms(text_t& query) {

  // we assume that for protemix, we don't have "high" unicode values.
  // don't do terms inside quotes
  text_t new_query="";

  unsigned int query_length=query.size();
  unsigned int i;
  bool inside_quotes=false;
  bool inside_field=false; // we need to OR together the terms inside fields
  bool in_a_term=false;
  text_t::const_iterator here = query.begin();
  text_t::const_iterator end = query.end();

  text_t::const_iterator term_start=here;
  // | and &...
  while (here < end) {
    if (*here=='"') {
      new_query.push_back('"');
      if (inside_quotes==false) {
        inside_quotes=true;
        while (*(++here) != '"')
          new_query.push_back(*here);
        new_query.push_back('"');
      } else {
        inside_quotes=false;
      }
    } else if (isspace(*here)) {
      if (in_a_term) { // we've reached the end of a term
        // this might include case and stem options...
        text_t term=substr(term_start,here);
        equiv_lookup(term);
        new_query+=term;
        new_query+=" ";
        in_a_term=false;
      }
    } else if (*here=='[') {
      // start of a field...
    } else if (*here==']') {
      // end of a field...
    } else if (!in_a_term) { // this char is start of term
      term_start=here;
      in_a_term=true;
    }

    ++here;
  }

  // tidy up for last term
  if (in_a_term) {
    text_t term=substr(term_start,here);
    equiv_lookup(term);
    new_query+=term;
  }
  query=new_query;
}
